package AntigoneUI.DB;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import java.util.*;
import java.sql.*;

//Own application classes
import AntigoneUI.*;

public class DBRequestManager implements RequestManager, HttpSessionBindingListener
{
	//default database setting value
	public static final String DB_NAME="AntigoneUI";
	private static final String USER="chuj";
	private static final String PASSWD="not2late";
		
	public String DB_DRIVER;
	public String DB_URL;
	public static final String REQUEST_HANDLER_PACKAGE="AntigoneUI.DB.requesthandlers";
	private Connection connection=null;
	private HashMap handlerHash=null;

	public DBRequestManager(String driver, String url) {
		DB_DRIVER=driver;
		DB_URL=url;
	}

	public String dispatchRequest(HttpServletRequest request) throws AppException {
		String action=request.getParameter("action");
		
		//Check if user is authenticated
		String user= (String) request.getSession().getAttribute("user");
		if (user==null && !action.equals("DBLoginRequestHandler") 
			&& !action.equals("DB_registration_RequestHandler")) 
		{
			throw new AppException("User is not authenticated to access database!");
		}

		//Loading the appropriate handler class
		DBRequestHandler handler=(DBRequestHandler) getHandlerInstance(action);
		
		handler.setConnection(connection);
		String result=handler.handleRequest(request);
		return (result);
	}
	
	public Connection getConnection() {
		return connection;
	}
	
	private RequestHandler getHandlerInstance(String action) 
		throws AppException
	{
		String handlerName=REQUEST_HANDLER_PACKAGE+"."+action;
		RequestHandler handler=(RequestHandler) handlerHash.get(handlerName);

		if (handler == null) {
			try {
				Class handlerClass=Class.forName(handlerName);
				if (!RequestHandler.class.isAssignableFrom(handlerClass) )
					throw new AppException("Class "+ handlerName +"do not implement RequestHandler Interface.");
				handler=(RequestHandler) handlerClass.newInstance();
				handlerHash.put(handlerName, handler);
			} catch (Exception e) {
				throw new AppException(e.getMessage()+"<br/>Loading DB RequestHandler class failed.");
			}
		}
		System.out.println("Finish loading: "+action);
		return handler;
	}
	
	public void valueBound(HttpSessionBindingEvent event ) {
		System.out.println("Binding DBRequestManager");
		handlerHash=new HashMap();
		
		//Establishing connection to database
		try {
			Class.forName(DB_DRIVER);
			connection=DriverManager.getConnection (DB_URL+DB_NAME, USER, PASSWD);			
		} catch (Exception e) {
			System.out.println(e.getMessage()+"\nCannot open database connection.");
		}
	}
	
	public void valueUnbound(HttpSessionBindingEvent event) {
		System.out.println("Unbinding DBRequestManager");
		if (connection!=null) {
			System.out.println("Closing DB connection.\n");
			try {
				connection.close();
			} catch (SQLException e) {}
		}
	}
	
}