package AntigoneUI.GUI;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import java.util.*;
import java.sql.*;

//Own application classes
import AntigoneUI.*;

public class GUIRequestManager implements RequestManager
{
	public static final String REQUEST_HANDLER_PACKAGE="AntigoneUI.GUI.requesthandlers";
	private HashMap handlerHash=null;

	public GUIRequestManager() {
		handlerHash=new HashMap();
	}

	public String dispatchRequest(HttpServletRequest request) throws AppException {
		String action=request.getParameter("action");
		
		//Check if user is authenticated
		String user= (String) request.getSession().getAttribute("user");
		if (user==null) {
			throw new AppException("User is not authenticated yet!");
		}

		//Loading the appropriate handler class
		GUIRequestHandler handler=(GUIRequestHandler) getHandlerInstance(action);
		
		String result=handler.handleRequest(request);
		return (result);
	}
	
	private RequestHandler getHandlerInstance(String action) 
		throws AppException
	{
		String handlerName=REQUEST_HANDLER_PACKAGE+"."+action;
		RequestHandler handler=(RequestHandler) handlerHash.get(handlerName);

		if (handler == null) {
			try {
				Class handlerClass=Class.forName(handlerName);
				if (!RequestHandler.class.isAssignableFrom(handlerClass) )
					throw new AppException("Class "+ handlerName +"do not implement RequestHandler Interface.");
				handler=(RequestHandler) handlerClass.newInstance();
				handlerHash.put(handlerName, handler);
			} catch (Exception e) {
				throw new AppException(e.getMessage()+"<br/>Loading GUI RequestHandler class failed.");
			}
		}
		System.out.println("Finish loading: "+action);
		return handler;
	}
}