package AntigoneUI.GUI.requesthandlers;

import java.io.*;
import java.util.*;

//3rd party classes
import org.dom4j.*;
import org.dom4j.io.*;

public class ViewTreeFactory
{
	class tmpObject
	{
		public String out;
		public int val;

		public tmpObject() {
			super();
			this.out="";
			this.val=0;
		}
			
		public tmpObject(String out, int val) {
			super();
			this.out=out;
			this.val=val;
		}
	}
	
	private File xmlFile;
	private final String LINKPREFIX="request?action=GUI_openPolicy_RequestHandler&ID=";
	private PrintWriter out;
	private	String bname="aux";
	
	public ViewTreeFactory (File xmlFile) {
		this.xmlFile=xmlFile;

		//Setup the output writer.
		try {
			File outfile=(new File(xmlFile.getParent()+File.separator+"leftTree.js") ).getCanonicalFile() ;
			out=new PrintWriter (new BufferedOutputStream( new FileOutputStream(outfile))) ;
		} catch (Exception e) {
			System.out.println("Cannot open leftTree.js");
			return;
		}
		
		BuildTreeStructure();
	}
	
	private void BuildTreeStructure() {
		out.println("USETEXTLINKS = 1");
		out.println("STARTALLOPEN = 0\n");
		
		//The first line of the tree
		String ID="root";
		out.println("foldersTree="+getFldStr("Policies", LINKPREFIX+ID, false, null) );

		//Open and read the XML file.
		Document document=null;
		try {
			SAXReader reader = new SAXReader();
			document = reader.read(xmlFile);
		} catch (Exception e) {
			System.out.println("Cannot read xmlFile.");
		}
		
		Element element=document.getRootElement();
		
		//System.out.println("Building policies tree");
		int cnt=0;
		String writeout="";
		//Building the 4 main branches: attributes, actions, provisionals, asserts

		
		//Building the attributes tree
		writeout=getFldStr("Attributes", LINKPREFIX+ID+"/attribute", true, "foldersTree");
		out.println(bname+(cnt++) + "="+writeout);
		cnt=BuildAttributesTree(document.getRootElement(), bname+(cnt-1), cnt, ID+"/attribute");
		

		//Building the provisional tree
		writeout=getFldStr("Provisionals", LINKPREFIX+ID+"/provisional", true, "foldersTree");
		out.println(bname+(cnt++) + "="+writeout);
		cnt=BuildProvisionalsTree(document.getRootElement(), bname+(cnt-1), cnt, ID+"/provisional");

		//Building the action tree
		writeout=getFldStr("Actions", LINKPREFIX+ID+"/action", true, "foldersTree");
		out.println(bname+(cnt++) + "="+writeout);
		cnt=BuildActionsTree(document.getRootElement(), bname+(cnt-1), cnt, ID+"/action");
		
		//Building assert tree
		/**
		  * Will be done soon.
		  **/
		out.flush();
		out.close();
	}
	
	private int BuildProvisionalsTree(Element element, String parent, int cnt, String ID) {
		//Building the provisionals tree
		int IDcnt=0;
		for (Iterator i=element.elementIterator ("provision"); i.hasNext(); ) {
			Element my_el = (Element) i.next();
			String tagname = my_el.attributeValue("tag");

			//Getting the policy list
			String writeout = getFldStr(tagname, LINKPREFIX+ID+"/clause"+IDcnt, true,parent);
			out.println( bname+(cnt++) +"="+writeout);
			
			//Building the children of that particular node
			cnt=recBuildProvisionals(my_el, cnt-1, "", ID+"/clause"+(IDcnt++) );
			out.println("");
			//Going to the next provision
		}
		return cnt;
	}
	
	/**
	*  The return value is the next counter value for building the tree.
	**/
	private int recBuildProvisionals (Element element, int parentCnt, String state, String ID) {
		int cnt=parentCnt+1;
		String parentStr=bname+parentCnt;
		String writeout="";
		
		//Extract the last part of the string to check the current state
		String lastState=null;
		if (state.lastIndexOf("_")!=-1) lastState=state.substring(state.lastIndexOf("_")+1);
		else lastState=state;
		
		 //On the 0th level of provisional clause
		if (lastState.equals("")) {

			//Doing the recursive conditionals here.
			boolean printCondStr=false;
			int IDcnt=0;
			for (Iterator i=element.elementIterator(); i.hasNext(); ) {
				Element child=(Element) i.next();
				if (child.getName().equals("predicate")) {
					if (!printCondStr) { //Printing conditional heading
						out.println(bname+(cnt++)+" = "+getLnkStr("Conditionals", "", true,parentStr, true) );
						printCondStr=true;
					}
					out.println(bname+(cnt++)+" = "+getFldStr(child.attributeValue("name"), LINKPREFIX+ID+"/predicate"+IDcnt, true, parentStr) );
					cnt=recBuildProvisionals(child, cnt-1, state+"provision", ID+"/predicate"+(IDcnt++) );
				}
			}
			
			//Consequences
			boolean printConseqStr=false;
			
			//Doing config here
			Iterator i=element.elementIterator("config");
			IDcnt=0;
			while (i.hasNext() ) {
				if (!printConseqStr) { //Printing consequence heading
					out.println(bname+(cnt++)+" = "+getLnkStr("Consequences", "", true,parentStr, true) );
					printConseqStr=true;
				}

				out.println(bname+(cnt++)+" = "+getFldStr("config", LINKPREFIX+ID+"/config"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildProvisionals(child, cnt-1, state+"config", ID+"/config"+(IDcnt++));
			}
			//Doing pick here
			i=element.elementIterator("pick");
			IDcnt=0;
			while (i.hasNext() ) {
				if (!printConseqStr) { //Printing consequence heading
					out.println(bname+(cnt++)+" = "+getLnkStr("Consequences", "", true,parentStr, true) );
					printConseqStr=true;
				}
				out.println(bname+(cnt++)+" = "+getFldStr("pick", LINKPREFIX+ID+"/pick"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildProvisionals(child, cnt-1, state+"pick", ID+"/pick"+(IDcnt++));
			}
			
			//Doing policy here
			boolean printPolStr=false;
			i=element.elementIterator("policy");
			IDcnt=0;
			while (i.hasNext() ) {
				if (!printPolStr) { //Printing policy heading
					out.println(bname+(cnt++)+" = "+getLnkStr("Tags", "", true,parentStr, true) );
					printPolStr=true;
				}
				Element child=(Element) i.next();
				String val=child.attributeValue("name");
				out.println(bname+(cnt++)+" = "+getLnkStr(val, LINKPREFIX+ID+"/policy"+(IDcnt++), true, parentStr, false));
			}
			return cnt;
		}
		
		//On the 1st level, inside provisional-conditional
		else if (lastState.equals("provision") ) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator("param"); i.hasNext(); ) {
				Element child=(Element) i.next();
				String value=child.attribute("value").getValue();
				out.println(bname+(cnt++)+" = "+getLnkStr(value, LINKPREFIX+ID+"/param"+(IDcnt++), true, parentStr, false));
			}
			
			return cnt;
		}

		else if (lastState.equals("pick")) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator("config"); i.hasNext();) {
				out.println(bname+(cnt++)+" = "+getFldStr("config", LINKPREFIX+ID+"/config"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildProvisionals(child, cnt-1, state+"_config", ID+"/config"+(IDcnt++));
			}
			return cnt;
		}
		//On the 1st level, inside provisional-config
		else if (lastState.equals("config") ) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator(); i.hasNext();) {
				Element child=(Element) i.next();
				out.println(bname+(cnt++)+" = "+getFldStr(child.attributeValue("name"), LINKPREFIX+ID+"/mechanism"+IDcnt, true, parentStr) );
				
				//Going into checking its parameter
				cnt=recBuildProvisionals(child, cnt-1, state+"_configparams", ID+"/mechanism"+(IDcnt++));
			}
			return cnt;
		}
		
		//On the 2nd level, inside provisional-config-mechanism
		else if (lastState.equals("configparams") ) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator(); i.hasNext(); ) {
				Element child=(Element) i.next();
				out.println(bname+(cnt++)+" = "+getLnkStr(child.attributeValue("name"), LINKPREFIX+ID+"/param"+(IDcnt++), true, parentStr, false) );
			}
			return cnt;
		}
		
		else return -1;
	}

	private int BuildActionsTree (Element element, String parentStr, int cnt, String ID) {
		int IDcnt=0;
		for (Iterator i=element.elementIterator("action"); i.hasNext(); ) {
			Element child=(Element) i.next();
			String tagname=child.attributeValue("tag");
			
			out.println(bname+(cnt++)+" = "+getFldStr(tagname, LINKPREFIX+ID+"/clause"+IDcnt, true, parentStr) );
			cnt=recBuildActions(child, cnt-1, "", ID+"/clause"+(IDcnt++));
			out.println("");
		}
		return cnt;
	}
	
	private int recBuildActions (Element element, int parentCnt, String state, String ID) {
		int cnt=parentCnt+1;
		String parentStr=bname+parentCnt;
		String writeout="";
		
		//Extract the last part of the string to check the current state
		String lastState=null;
		if (state.lastIndexOf("_")!=-1) lastState=state.substring(state.lastIndexOf("_")+1);
		else lastState=state;
		
		 //On the 0th level of provisional clause
		if (lastState.equals("")) {
			
			//Doing the recursive conditionals here.
			int IDcnt=0;
			int flag=0;
			for (Iterator i=element.elementIterator(); i.hasNext(); ) {
				Element child=(Element) i.next();
				if (child.getName().equals("predicate")) {
					if (flag==0) {
						flag=1;
						out.println(bname+(cnt++)+" = "+getLnkStr("Conditionals", "", true,parentStr, true) );
					}
					out.println(bname+(cnt++)+" = "+getFldStr(child.attributeValue("name"), LINKPREFIX+ID+"/provision"+IDcnt, true, parentStr) );
					cnt=recBuildActions(child, cnt-1, state+"provision", ID+"/provision"+(IDcnt++));
				}
			}
			//Doing credential here
			Iterator i=element.elementIterator("credential");
			IDcnt=0;
			flag=0;
			while (i.hasNext() ) {
				if (flag==0) {
					flag=1;
					out.println(bname+(cnt++)+" = "+getLnkStr("Credentials", "", true,parentStr, true) );
				}
				
				out.println(bname+(cnt++)+" = "+getFldStr("credential", LINKPREFIX+ID+"/credential"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildActions(child, cnt-1, state+"credential", ID+"/credential"+(IDcnt++));
			}
			
			//Doing config here
			i=element.elementIterator("config");
			IDcnt=0;
			flag=0;
			while (i.hasNext() ) {
				if (flag==0) {
					flag=1;
					out.println(bname+(cnt++)+" = "+getLnkStr("Config & Pick", "", true,parentStr, true) );
				}

				out.println(bname+(cnt++)+" = "+getFldStr("config", LINKPREFIX+ID+"/config"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildActions(child, cnt-1, state+"config", ID+"/config"+(IDcnt++));
			}
			//Doing pick here
			i=element.elementIterator("pick");
			IDcnt=0;
			while (i.hasNext() ) {
				if (flag==0) {
					flag=1;
					out.println(bname+(cnt++)+" = "+getLnkStr("Config & Pick", "", true,parentStr, true) );
				}

				out.println(bname+(cnt++)+" = "+getFldStr("pick", LINKPREFIX+ID+"/pick"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildActions(child, cnt-1, state+"pick", ID+"/pick"+(IDcnt++));
			}
			return cnt;
		}
		
		else if (lastState.equals("credential")) {
			//getting the binding variable
			Iterator i=element.elementIterator("bindvar");
			if (i.hasNext()) {
				Element child=(Element) i.next();
				out.println(bname+(cnt++)+" = "+getLnkStr("Bind Variable", "", true, parentStr, true) );
				out.println(bname+(cnt++)+" = "+getLnkStr(child.attributeValue("name"), LINKPREFIX+ID+"/bindvar", true, parentStr, false) );
			}
			
			//getting the credential argument
			i=element.elementIterator();
			boolean printCArgStr=false;
			int IDcnt=0;
			while(i.hasNext() ) {
				Element child=(Element) i.next();
				if (child.getName().equals("cred_arg")) {
					if (!printCArgStr) { //Printing conditional heading
						out.println(bname+(cnt++)+" = "+getLnkStr("Credential Argument", "", true,parentStr, true) );
						printCArgStr=true;
					}
					out.println(bname+(cnt++)+" = "+getLnkStr(child.attributeValue("name"), LINKPREFIX+ID+"/arg"+(IDcnt++), true, parentStr, false) );
				}
			}
			return cnt;
		}
		
		//On the 1st level, inside provisional-conditional
		else if (lastState.equals("provision") ) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator("param"); i.hasNext(); ) {
				Element child=(Element) i.next();
				String value=child.attribute("value").getValue();
				out.println(bname+(cnt++)+" = "+getLnkStr(value, LINKPREFIX+ID+"/param"+(IDcnt++), true, parentStr, false));
			}
			return cnt;
		}
		
		else if (lastState.equals("pick")) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator("config"); i.hasNext();) {
				out.println(bname+(cnt++)+" = "+getFldStr("config", LINKPREFIX+ID+"/pick"+IDcnt, true, parentStr) );
				Element child=(Element) i.next();
				cnt=recBuildActions(child, cnt-1, state+"_config",ID+"/pick"+(IDcnt++));
			}
			return cnt;
		}
		//On the 1st level, inside provisional-config
		else if (lastState.equals("config") ) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator(); i.hasNext();) {
				Element child=(Element) i.next();
				out.println(bname+(cnt++)+" = "+getFldStr(child.attributeValue("name"), LINKPREFIX+ID+"/mechanism"+IDcnt, true, parentStr) );
				
				//Going into checking its parameter
				cnt=recBuildActions(child, cnt-1, state+"_configparams", ID+"/config"+(IDcnt++));
			}
			return cnt;
		}
		
		//On the 2nd level, inside provisional-config-mechanism
		else if (lastState.equals("configparams") ) {
			int IDcnt=0;
			for (Iterator i=element.elementIterator(); i.hasNext(); ) {
				Element child=(Element) i.next();
				out.println(bname+(cnt++)+" = "+getLnkStr(child.attributeValue("name"), LINKPREFIX+ID+"/param"+(IDcnt++), true, parentStr, false) );
			}
			return cnt;
		}

		else return -1;
	}

	private int BuildAttributesTree(Element element, String parent, int cnt, String ID) {
		//System.out.println("Building attribute tree.");
		int IDcnt=0;
		for (Iterator i=element.elementIterator("attribute"); i.hasNext(); ) {
				
			Element my_el = (Element) i.next();
			String tagname = my_el.attributeValue("tag");
				
			//The first line of each attribute list.
			String writeout = getFldStr (tagname, LINKPREFIX+ID+"/tag"+IDcnt, true, parent );
			out.println( bname+(cnt) +"="+writeout);
			cnt++;
				
			//Get all the parameter of that attribute
			writeout=getParams(my_el, (bname+(cnt-1) ), ID+"/tag"+IDcnt);
			out.println(writeout);
			
			boolean hasChild=(my_el.elementIterator("set").hasNext()?true:false);
				
			//Get all the subsets and the parameters of those subsets.
			writeout="";
			if (hasChild) {
				tmpObject obj=recBuildSet(cnt, bname+(cnt-1), my_el, tagname, ID+"/tag"+IDcnt);
				cnt=obj.val;
				out.println(obj.out);
			}
			IDcnt++;
		}
		return cnt;
	}
	
	private String getParams(Element my_el, String parent, String ID) {
		String output="";
		int IDcnt=0;
		for (Iterator i=my_el.elementIterator("param") ; i.hasNext();) {
			String val = ( (Element) i.next() ).attributeValue("value");
			output+= getLnkStr (val, LINKPREFIX+ID+"/param"+(IDcnt++), true, parent, false)+"\n"; 
		}
		return output;
	}
	
	//String out parameter is modified to include the built set.
	private tmpObject recBuildSet (int cnt, String parent, Element element, String destPre, String ID) {
		String out="";
		int destCnt=0;
		int IDcnt=0;
		for ( Iterator i=element.elementIterator ("set"); i.hasNext(); destCnt++) {
			String bname="aux"+cnt;
			cnt++;

			out += ( bname+ " = " + getFldStr ("Set", LINKPREFIX+ID+"/set"+IDcnt, true, parent)+"\n" );

			Element my_el=(Element) i.next();
			//Get the parameter of that set
			out+=getParams(my_el, bname, ID+"/set"+IDcnt);

			//System.out.println( getParams(my_el, bname) );

			boolean hasChild = (my_el.elementIterator("set").hasNext()?true:false);
			if (hasChild) {
				tmpObject my_obj=recBuildSet (cnt, bname, my_el, destPre+"_"+destCnt, ID+"/set"+IDcnt);
				cnt=my_obj.val;
				out+=my_obj.out;
			}
			IDcnt++;
		}
		tmpObject obj=new tmpObject (out, cnt);
		return obj;
	}
	
	private String getFldStr (String text, String link, boolean insert, String parent) {
		if (insert==false) return ("gFld (\""+text+"\", \""+link+"\" );" );
		else return ("insFld("+parent+", gFld (\""+text+"\", \""+link+"\" ) );");
	}
	
	private String getLnkStr (String text, String link, boolean insert, String parent, boolean heading) {
		if (insert==false) return ("gLnk(2, \""+text+"\", \""+link+"\"," + (heading? "1":"0")+");" );
		else return ("insDoc("+parent+", gLnk(2, \""+text+"\", \""+link+"\","+ (heading? "1":"0") +") );");
	}
}