/**
 * Purpose:	A conversion tool to convert file from Ismene policy file to XML based file.
 * Author: 	Jimmy Chu
 * Version: 	1.0
 * Acknowledgment: Thanks God for giving me the strength and wisdom in writing this file.
 */
package Converter;

import java.io.*;
import java.util.*;
import Converter.*;

public class Conversion
{
	//Instance Variables
	BufferedReader readin;
	BufferedWriter writeout;
	
	int numtab=0;
	int handlingcomment=0;
	
	
	//Main function to activate the conversion.
	public static void main (String args[]) {
		Conversion conversion=new Conversion(args);
	}
	
	//Constructor
	public Conversion (String args[]) {
		//local variables Declaration
		char chr=0;
		boolean ending=false;
		String sentence=null;
		String temp=null;
		

		
		if (args.length!=2) {
			System.out.println("Usage: java Conversion <Ismene input filename> <XML output filename>.");
			System.exit(0);
		}
		
		//Creating the buffer input filestream
		try {
			readin=new BufferedReader(new FileReader(args[0]));
		} catch (Exception e) {
			System.out.println("Ismene input file not found.\n");
			System.exit(0);
		}
		
		//Creating the output filestream
		try {
			writeout=new BufferedWriter(new FileWriter (new File(args[1])) );
		} catch (Exception e) {}
		
		try {
			WriteString("<?xml version=\"1.0\"?>\n", true);
			//WriteString("<?xml-stylesheet href=\"conversion.xsl\" type=\"text/xsl\"?>\n", true);
			WriteString("<XMLpolicies>\n", true);
			do {
				//Reading in a line
				sentence=null;
				do {
					temp=readin.readLine();

					if (temp!=null) {
						temp=temp.trim();
						if (sentence==null) sentence=temp;
						else sentence+=temp;
					}
					
					//Deciding which category the sentence belong to.
					if (sentence.indexOf('%')!=-1) {
						//Handle the comment and cut that part out of the sentence.
						Handle_Comment(sentence);
						handlingcomment=1;
						sentence=sentence.substring(0, sentence.indexOf('%'));
					}
				} while (temp!=null && sentence.indexOf(';')==-1);
				
				if (sentence!=null) {
				/*
				chr=(char) readin.read();
				
				//Check end of file
				if ( (short)chr==(short)(-1) ) ending=true;
				//Excluding the last -1 value.
				if (ending==false) sentence=sentence+chr; 
				if (chr ==';' || ending==true || (chr=='\n' //carriage return
				&& (sentence.indexOf('%')!=-1 ))) */
					
					handlingcomment=0;
//				if (sentence.indexOf("assert")!=-1) Handle_Assert(sentence);
					if (sentence.indexOf("accept")!=-1) Handle_Action(sentence);
					else if (sentence.indexOf(":=")!=-1) Handle_Attribute(sentence);
					else if ((sentence.indexOf(":")!=-1) && (sentence.indexOf("::")!=-1)) Handle_Provision(sentence);
					//Delete the sentence.
				}
			} while (sentence!=null);
		} catch (Exception e) {}
		
		WriteString("</XMLpolicies>\n", true);
		
		//Closing the file stream
		try {
			readin.close();
			writeout.close();
		} catch (Exception e) {}
	}

	void Handle_Attribute(String string) {
		String name=null, content=null, token=null, readin=null;
		int pos=0;
	
		name=string.substring(0, string.indexOf(":=")).trim();
		numtab++;
		WriteString("\n", false);
		WriteString("<attribute tag=\""+name+"\">\n", true);
		content=string.substring(string.indexOf("<")+1, string.indexOf(">")+1);
		StringTokenizer strtok=new StringTokenizer(content, TS_Constants.ATTR_DELIM, true);
		
		do {
			token=getNextNSToken(strtok).trim();
			if (token==null) break;
			
			if (token.equals("<")) {}
			else if (NotDelim(token, TS_Constants.ATTR_DELIM)) {
				pos=1;
				readin=new String(token);
			}
			else if (token.equals(",") || token.equals(">") || token.equals("}")) {
				if (pos==1) {
					pos=0;
					numtab++;
					WriteString("<param value=\""+readin+"\"/>\n", true);
					numtab--;
				}
                if (token.equals("}")) {
         			WriteString("</set>\n", true);
				    numtab--;       
                }
			}
			else if (token.equals("=")) {
				pos=0;
				Handle_AE(strtok, readin, TS_Constants.ATTR_VAR);
			}
			else if (token.equals("{")) {
				numtab++;
				WriteString("<set>\n", true);
			}
		} while (!token.equals(">"));

		WriteString("</attribute>\n", true);
		numtab--;
	}
	
	/**
	 *To manage the writeout of <comment> block.
	 */
	void Handle_Comment(String string) {
		numtab++;
		WriteString("<comment content=\""+string.substring(string.indexOf('%')+1).trim()+"\"/>\n", true);
		numtab--;
	}
	
	void Handle_Action(String string) {
		String token=null, readin=null;
		String name=null, condition=null;
		int posflag=TS_Constants.UNDEFINED;
		
		name=string.substring(0, string.indexOf(":")).trim();
		numtab++;
		WriteString("\n", false);
		WriteString("<action tag=\""+name+"\" reconfig=\"", true);
		if (string.indexOf("reconfig")!=-1) WriteString("true\">\n", false);
		else WriteString("false\">\n", false);
		
		condition=string.substring(string.indexOf(":")+1, string.indexOf("::")).trim();
		StringTokenizer strtok=new StringTokenizer (condition, TS_Constants.DELIM, true);
		do {
			token = getNextNSToken(strtok);
			if (token==null) break;
			
			if (NotDelim(token, TS_Constants.DELIM)) {
				readin=new String(token);
				if (readin.equalsIgnoreCase("config")) posflag=TS_Constants.CONFIG;
				else if (readin.equalsIgnoreCase("pick")) posflag=TS_Constants.PICK;
				else if (readin.equalsIgnoreCase("credential")) posflag=TS_Constants.CREDENTIAL;
				else posflag=TS_Constants.CONDITIONALS;
			}
			else if (token.equals("("))
				Handle_Function(strtok, readin, posflag);
			else if (token.equals("=")) 
				Handle_AE(strtok, readin, TS_Constants.VAR);
		} while (strtok.hasMoreTokens());
		
		WriteString("</action>\n", true);
		numtab--;
	}
	
	void Handle_Provision(String sentence) {
		//Local Variables
		String tag=sentence.substring(0, sentence.indexOf(":")).trim();
		String conditional=sentence.substring(sentence.indexOf(":")+1, sentence.indexOf("::")).trim();
		String consequence=sentence.substring(sentence.indexOf("::")+2).trim();
		String token=null;
		String name=null;
		int posflag=0;
		
		numtab++;
		WriteString("\n", false);
		WriteString("<provision tag=\""+tag+"\">\n", true);
		
		//Handling the conditional side.
		StringTokenizer strtok=new StringTokenizer(conditional, TS_Constants.DELIM, true);
		while (strtok.hasMoreTokens()) {
			token=getNextNSToken(strtok);
			if (token==null) break;
			
			if (token.equals("=")) {
				Handle_AE(strtok, name, TS_Constants.VAR);
			}
			else if (token.equals("(")) {
				Handle_Function(strtok, name, TS_Constants.CONDITIONALS);
			}
			else if (NotDelim(token, TS_Constants.DELIM)) {
				name=new String(token);
			}
		}
		
		//Handle the consequence side.
		strtok=new StringTokenizer (consequence, TS_Constants.DELIM, true);
		while (strtok.hasMoreTokens()) {
			token=getNextNSToken(strtok);
			if (token==null) break;
			
			if ( (token.equals(",")|| token.equals(";")) && posflag==TS_Constants.POLICY) {
				numtab++;
				WriteString("<policy name=\""+name+"\"/>\n", true);
				numtab--;
			}
			else if (token.equals("(")) {
				Handle_Function(strtok, name, posflag);
			}
			else if (NotDelim(token, TS_Constants.DELIM)) {
				name=new String(token);
				if (name.equalsIgnoreCase("config")) posflag=TS_Constants.CONFIG;
				else if (name.equalsIgnoreCase("pick")) posflag=TS_Constants.PICK;
				else posflag=TS_Constants.POLICY;
			}
		}
		WriteString("</provision>\n", true);
		numtab--;
	}
		
	void Handle_Function(StringTokenizer strtok, String name, int posflag) {
		String token=null;
		String readin=null; //Similar to the name var in the above variable.
		int nextposflag=0;
		int change=0;
		
		//StringTokenizer strtok, String name, int posflag
		
		if (posflag==TS_Constants.CONDITIONALS) {
			numtab++;
			WriteString("<predicate name=\""+name+"\"", true);
			do {
				token=getNextNSToken(strtok);
				if (token==null) break;
				
				if ((token.equals(",") || token.equals(")")) && change==1)
					Handle_AE(strtok, readin, TS_Constants.PREDICATE);
				else if (NotDelim(token, TS_Constants.DELIM)) {
					if (change==0) {
						change=1;
						WriteString(">\n", false);
					}
					readin=new String(token);
				}
			} while (token.equals(")")==false);
			
			if (change==0) WriteString("/>\n", false);
			else WriteString("</predicate>\n", true);
			numtab--;
		}
		else if (posflag==TS_Constants.CONFIG) {
			numtab++;
			WriteString("<config", true);
			do {
				token=getNextNSToken(strtok);
				if (token==null) break;
				
				if ((token.equals(")") || token.equals(",")) && (change==1) ) {
					numtab++;
					WriteString("<mechanism/>\n", true);
					numtab--;
				}
				else if (token.equals("(")) {
					Handle_Function(strtok, readin, TS_Constants.MECHANISM);
					change=2;
				}
				else if (NotDelim(token, TS_Constants.DELIM)) {
					if (change==0) {
						change=1;
						WriteString(">\n", false);
					}
					readin=new String(token);
				}
				
			} while (token.equals(")")==false); 
			
			
			if (change>0) WriteString("</config>\n", true);
			else WriteString("/>\n", false);
			numtab--;
		}
		else if (posflag==TS_Constants.PICK) {
			int config_no=0;
			numtab++;
			WriteString("<pick>\n", true);
			do {
				token=getNextNSToken(strtok);
				if (token==null) break;
				
				if (NotDelim(token, TS_Constants.DELIM)) {
					readin=new String(token);
					if (readin.equalsIgnoreCase("config")) config_no++;
				}
				else if (token.equals("(")) 
					Handle_Function(strtok, readin, TS_Constants.CONFIG);
			}	while (token.equals(")")==false);
			WriteString("</pick>\n", true);
			numtab--;
		}
		else if (posflag==TS_Constants.MECHANISM) {
			numtab++;
			WriteString("<mechanism name=\""+name+"\"", true);
			do {
				token=getNextNSToken(strtok);
				if (token==null) break;
				
				if (NotDelim(token, TS_Constants.DELIM)) {
					if (change==0) {
						change=1;
						WriteString(">\n", false);
					}
					readin=new String(token);
				}
				else if (token.equals("="))
					token=Handle_AE(strtok, readin, TS_Constants.MECHANISM_PARAM);
			} while (token.equals(")")==false);
			
			if (change>0) WriteString("</mechanism>\n", true);
			else WriteString("/>\n", false);
			numtab--;
		}
		
		else if (posflag==TS_Constants.CREDENTIAL) {
			numtab++;
			WriteString("<credential", true);
			do {
				token=getNextNSToken(strtok);
				if (token==null) break;
				
				if (NotDelim(token, TS_Constants.DELIM)) {
					if (change==0) {
						change=1;
						WriteString(">\n", false);
					}
					readin=new String(token);
				}
				else if (token.equals(",")) {
					if (change==1) {
						change=2;
						numtab++;
						WriteString("<bindvar name=\""+readin+"\"/>\n", true);
						numtab--;
					}
				}
				else if (token.equals("="))
					token=Handle_AE(strtok, readin, TS_Constants.CREDENTIAL);
				else if (token.equals(")")) {
					if (change==1) {
						numtab++;
						WriteString("<bindvar name=\""+readin+"\"/>\n", true);
						numtab--;
					}
				}
					
			} while (!token.equals(")"));
			
			if (change>0) WriteString("</credential>\n", true);
			else WriteString("/>\n", false);
			numtab--;
		}
	}
	
	//Handle assign or equal sign
	String Handle_AE(StringTokenizer strtok, String name, int posflag) {
		String token=null;
		
		if (posflag==TS_Constants.VAR) {
			token=getNextNSToken(strtok);
			
			numtab++;
			WriteString("<var name=\""+name+"\" value=\""+token+"\"/>\n", true);
			numtab--;
			
		} else if (posflag==TS_Constants.PREDICATE) {
			numtab++;
			WriteString("<param value=\""+name+"\"/>\n", true);
			numtab--;
			
		} else do {
			token=getNextNSToken(strtok);
			if (token==null) break;
			
			if (NotDelim(token, TS_Constants.DELIM)) {
				numtab++;
				if (posflag==TS_Constants.MECHANISM_PARAM)
					WriteString("<mechParam name=\""+name+"\" value=\""+token+"\"/>\n", true);
				else WriteString("<cred_arg name=\""+name+"\" value=\""+token+"\"/>\n", true);
				numtab--;
			}
		} while ( (token.equals(",")==false) && (token.equals(")")==false) );
		return token;
	}
	
	/**
		To write out a string to output stream.
	*/
	void WriteString(String string, boolean printtab) {
		int index=0;
		//Write out the tab for alignment.
		if (printtab) {
			for (int i=0;i<numtab*4;i++) {
				try {
					writeout.write(" ", 0, 1);
				} catch (Exception e) {}
			}
		}
		//for (int i=0;i<string.length();i++) {
			try {
				while (  ( index=string.indexOf('&', index)) !=-1 ) {
					StringBuffer strbuf=new StringBuffer(string);
					strbuf=strbuf.insert(index+1, "amp;");
					index+=5;
				}
				writeout.write(string, 0, string.length() );
			} catch (Exception e) {}
		
	}

	String getNextNSToken(StringTokenizer strtok) {
		String token;
		while (strtok.hasMoreTokens()) {
			token=strtok.nextToken().trim();
			if (token.length()>0) return token;
		}
		return null;
	}
	
	boolean NotDelim(String string, String delim) {
		for (int i=0;i<delim.length();i++) {
			if (delim.charAt(i)==string.charAt(0)) return false;
		}
		return true;
	}
}
