/**
 * Purpose: A conversion tool to convert file from XML to Ismene file.
 * Author: Jimmy Chu
 * Version:1.0
 *Acknowledgment: Thanks God for giving me the strength and wisdom in writing this file.
 */
package Converter;

import java.io.*;
import java.util.*;

//third-party class
import org.dom4j.*;
import org.dom4j.io.*;

public class toIsmene
{
	//Instance Variables
	BufferedReader readin;
	BufferedWriter writeout;

	public static void main (String args[]) {
		try {
			toIsmene toismene=new toIsmene(args);
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
	public toIsmene (String args[]) throws IOException {
		if (args.length!=2) {
			System.out.println("Usage: java toIsmene <XML input file> <Ismene output file>.");
			System.exit(0);
		}
			
		//Opening the inputfile buffer reader
		try {
			readin= new BufferedReader(new FileReader(args[0]));
		} catch (Exception e) {
			System.out.println("XML input file open error.\n");
			System.exit(0);
		}
		
		//Using dom4j for those empty tags make them to have a closing tag
		Document document=null;
		try {
			SAXReader reader = new SAXReader();
			document = reader.read(args[0]);
		} catch (Exception e) {
			System.out.println("Cannot read xmlFile.");
		}
		Element root=document.getRootElement();
		
		//Opening the outputfile buffer writer
		try {
			writeout=new BufferedWriter(new FileWriter(new File(args[1])) ) ;
		} catch (Exception e) {
			System.out.println("Ismene output file open error.\n");
			System.exit(0);
		}
		
		for (Iterator it=root.elementIterator(); it.hasNext();) {
			Element element=(Element) it.next();
			
			if (element.getName().equals("comment")) handleComment(element);
			else if (element.getName().equals("attribute")) handleAttribute(element);
			else if (element.getName().equals("provision")) handleProvision(element);
			else if (element.getName().equals("action")) handleAction(element);
		}
		
		System.out.println("End.");
		writeout.close();
	}
	
	void handleComment(Element element) throws IOException {
		writeout.write("%"+element.attributeValue("content")+"\n");
	}
	
	void handleAttribute(Element element) throws IOException {
		writeout.write(element.attributeValue("tag")+" := <");
		recHandleAttribute(element);
		writeout.write(">;\n");
	}
	
	void recHandleAttribute(Element element) throws IOException {
		boolean comma=false;
		for (Iterator it=element.elementIterator(); it.hasNext();) {
			Element child=(Element) it.next();
			if (child.getName().equals("param")) {
				writeout.write( (comma==true?", ":"")+ child.attributeValue("value"));
				comma=true;
			} else {
				writeout.write( (comma==true?", ":"")+"{");
				recHandleAttribute(child);
				comma=true;
				writeout.write("}");
			}
		}
	}
	
	void handleProvision(Element element) throws IOException {
		boolean comma=false;
		Iterator it=null;
		
		writeout.write(element.attributeValue("tag")+": ");
		handleConditional(element);
		writeout.write("::");
		
		comma=false;
		it=element.elementIterator("pick"); 
		if (it.hasNext()) {
			handlePick(element);
			comma=true;
		}
		
		it=element.elementIterator("config");
		if (it.hasNext()) {
			writeout.write(comma==true?", ":"");
			handleConfig(element);
			comma=true;
		}
		for (it=element.elementIterator("policy"); it.hasNext();) {
			writeout.write ((comma==true?", ":"")+((Element) it.next()).attributeValue("name") );
			comma=true;
		}
		
		writeout.write(";\n");
	}
	
	void handleConditional (Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("var"); it.hasNext();) {
			Element child=(Element) it.next();
			writeout.write((comma==true?", ":"")+child.attributeValue("name")+"="+child.attributeValue("value"));
			comma=true;
		}
		
		for (Iterator it=element.elementIterator("predicate"); it.hasNext();) {
			Element child=(Element) it.next();
			writeout.write((comma==true?", ":"")+child.attributeValue("name")+" (");
			handlePredicateParam(child);
			writeout.write(")");
			comma=true;
		}
	}
	
	void handlePredicateParam(Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("param"); it.hasNext();) {
			Element child=(Element) it.next();
			writeout.write((comma==true?", ":"")+child.attributeValue("value"));
			comma=true;
		}
	}
	
	void handlePick(Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("pick"); it.hasNext();) {
			Element child=(Element) it.next();
			
			writeout.write( (comma==true?", ":"")+"pick(");
			handleConfig(child);
			writeout.write(")");
			comma=true;
		}
	}
	
	void handleConfig(Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("config"); it.hasNext();) {
			Element child=(Element) it.next();
			
			writeout.write( (comma==true?", ":"")+"config(");
			handleConfigMechanism(child);
			writeout.write(")");
			comma=true;
		}
	}
	
	void handleConfigMechanism(Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("mechanism"); it.hasNext();) {
			Element child=(Element) it.next();
			
			writeout.write( (comma==true?", ":"")+child.attributeValue("name")+"(");
			handleMechanismParam(child);
			writeout.write(")");
			comma=true;
		}
	}
	
	void handleMechanismParam(Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("mechParam"); it.hasNext();) {
			Element child=(Element) it.next();
			
			writeout.write( (comma==true?", ":"")+child.attributeValue("name")+"="+child.attributeValue("value"));
			comma=true;
		}
	}
	
	void handleAction(Element element) throws IOException {
		boolean comma=false;
		Iterator it=null;
		
		writeout.write(element.attributeValue("tag")+": ");
		
		for (it=element.elementIterator(); it.hasNext();) {
			Element child=(Element) it.next();
			if (child.getName().equals("predicate") || child.getName().equals("var")) {
				handleConditional(element);
				comma=true;
				break;
			}
		}
		
		it=element.elementIterator("pick"); 
		if (it.hasNext()) {
			writeout.write(comma==true?", ":"");
			handlePick(element);
			comma=true;
		}
		
		it=element.elementIterator("config");
		if (it.hasNext()) {
			writeout.write(comma==true?", ":"");
			handleConfig(element);
			comma=true;
		}

		//credential at here
		it=element.elementIterator("credential");
		if (it.hasNext()) {
			writeout.write(comma==true?", ":"");
			handleCredential(element);
			comma=true;
		}
		
		writeout.write("::accept"+(element.attributeValue("reconfig").equals("true")?", reconfig":"")+";\n");
	}
	
	void handleCredential (Element element) throws IOException {
		boolean comma=false;
		boolean subComma=false;
		
		for (Iterator it=element.elementIterator("credential"); it.hasNext(); ) {
			Element child=(Element) it.next();
			
			writeout.write((comma==true?", ":"")+"Credential (");			
			Iterator subIt=child.elementIterator("bindvar");
			if (subIt.hasNext() ) {
				writeout.write( ((Element) subIt.next()).attributeValue("name"));
				subComma=true;
			}	
			
			subIt=child.elementIterator("cred_arg");
			
			if (subIt.hasNext()) {
				writeout.write((subComma==true?", ":""));
				handleCredentialArg(child);
			}
			
			writeout.write(")");			
			comma=true;
		}
	}
	
	void handleCredentialArg(Element element) throws IOException {
		boolean comma=false;
		
		for (Iterator it=element.elementIterator("cred_arg"); it.hasNext(); ) {
			Element child=(Element) it.next();
			writeout.write( (comma==true?", ":"")+child.attributeValue("name")+"="+child.attributeValue("value"));
			comma=true;
		}
	}
}